const express = require('express');
const bodyParser = require('body-parser');
const app = express();
const cors = require('cors');
const userRoute = require('./routers/userRouter');
const chatRoute = require('./routers/chatRouter');
const messageRoute = require('./routers/messageRouter');
const adminRoute = require('./routers/AdminRouter');
const companyRoute = require('./routers/CompanyRouter');
const Message = require('./models/Message');
const CryptoJS = require("crypto-js");
const fileUpload = require('express-fileupload');
const https = require('https');
const { Op } = require("sequelize");
const { Server } = require("socket.io");
const { sequelize } = require('./connection');
const Group = require('./models/Group');
const GroupMember = require('./models/GroupMembers');
const User = require('./models/User');
const MessageRecipient = require('./models/MessageRecipient');

const fs = require('fs');

const options = {
    key: fs.readFileSync('api_hidemychat_llc.key'), // replace it with your key path
    cert: fs.readFileSync('api_hidemychat_llc.crt'), // replace it with your certificate path
}

const server = https.createServer(options,app);

const io = new Server(server, {
    transports: ['websocket'],
    cors: {
        origin: ["https://hidemychat.llc", "http://localhost:3000", "http://127.0.0.1:3045", "http://hidemychat.llc"],
        methods: ["GET", "POST"]
    }
});

const PORT = process.env.PORT || 4001;
const APP_URL = process.env.API_URL || 4001;
const key = "mySecretKey123"; // 32 bytes = 256 bits

let onlineUsers = [];

app.use(cors());
app.use(fileUpload());
app.use(bodyParser.json());

app.get('/test', (req, res) => {

    res.json({
        success: 0,
        message: 'File Size is to Large, maximum file size allowd 5MB'
    });
})
app.get('/', (req, res) => {

    res.json({
        success: 0,
        message: 'Not Found'
    });
})


app.use((req, res, next) => {
    res.setHeader('Access-Control-Allow-Origin', '*');
    res.setHeader('Access-Control-Allow-Methods', 'GET, POST, PUT, DELETE');
    res.setHeader('Access-Control-Allow-Headers', 'Content-Type, Authorization');
    next();
});
app.use(cors({
    origin: ["https://hidemychat.llc/", "http://localhost:3045", "http://127.0.0.1:3045", "http://hidemychat.llc/"]
}));
// call routes
app.use("/users", userRoute);
app.use("/chats", (req, res, next) => {
    req.io = io; // Pass the io object to the request object
    next();
}, chatRoute);
app.use("/messages", messageRoute);
app.use("/company", (req, res, next) => {
    req.io = io; // Pass the io object to the request object
    req.onlineUsers = onlineUsers; // Pass the io object to the request object
    next();
}, companyRoute);

// admin user route
app.use("/admin", adminRoute);


// Serve static files
app.use("/images", express.static('upload/images'));

app.use("/files", express.static('upload/files'));

// Serve static files
app.use("/assets", express.static('assets'));


// POST route for uploading image
app.post("/upload", (req, res) => {
    const file = req.files.image;
    const fileSize = req.files.image.size;

    if (fileSize < (5 * 1024 * 1024)) {
        let uploadPath = 'upload/images/' + file.name;
        file.mv(uploadPath, function (err) {
            if (err) {
                return res.status(500).json({
                    success: 0,
                    message: 'Error uploading file',
                    error: err.message // Include error message in the response
                });
            }
            const imageUrl = `/images/${file.name}`; // Change URL as needed
            // Here you can save imageUrl to your database or do further processing
            res.json({
                success: 1,
                imageUrl: imageUrl,
                size: fileSize.size,
                type: file.mimetype,
            });
        });
    }
    else {
        res.json({
            success: 0,
            message: 'File Size is to Large, maximum file size allowd 5MB'
        });
    }
});


// POST route for uploading image
app.post("/file/upload", (req, res) => {

    const fileSize = req.files.file.size;

    if (fileSize < (5 * 1024 * 1024)) {
        const randomName = generateRandomString(20);
        const file = req.files.file;
        const fileSize = req.files.file.size;
        const fileExtension = file.name.slice((file.name.lastIndexOf('.') - 1 >>> 0) + 2);

        if (fileSize < (5 * 1024 * 1024)) {
            let uploadPath = 'upload/files/' + randomName + '.' + fileExtension;
            file.mv(uploadPath, async function (err) {
                if (err) {
                    return res.status(500).json({
                        success: 0,
                        message: 'Error uploading file',
                        error: err.message // Include error message in the response
                    });
                }
                const fileName = '/files/' + randomName + '.' + fileExtension;

                const fileData = {
                    file_path: fileName,
                    file_name: file.name,
                    file_size: file.size,
                    file_type: file.mimetype,
                }
                return res.status(200).json(fileData);
            });
        }
    }
    else {
        res.json({
            success: 0,
            message: 'File Size is to Large, maximum file size allowd 5MB'
        });
    }

});


const encryptText = (text, key) => {
    const encrypted = CryptoJS.AES.encrypt(text, key).toString();
    return encrypted;
}

const decryptText = (encryptedText) => {
    try {
      const decrypted = CryptoJS.AES.decrypt(encryptedText, key).toString(CryptoJS.enc.Utf8);
      return decrypted
    } catch (error) {
      console.error("Decryption error:", error);
      return error
    }
};


const generateRandomString = (length) => {
    const characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    let result = '';
    for (let i = 0; i < length; i++) {
        result += characters.charAt(Math.floor(Math.random() * characters.length));
    }
    return result;
};
// Serve static files
app.use("/documents", express.static('upload/documents'));



//creates onlineUsers Array
const activeSessions = {}; // { userId: socketId }

//connection of socket.io
io.on("connection", async (socket) => {
    const userId = socket.handshake.query.userId;
     if (activeSessions[userId]) {
        // Disconnect the previous session
        const previousSocketId = activeSessions[userId];
        io.to(previousSocketId).emit('force_logout');
        io.sockets.sockets.get(previousSocketId)?.disconnect(true);
    }
    // Save the new session
    activeSessions[userId] = socket.id;
    
    socket.join(userId);

    const group = await GroupMember.findAll({ where: { user_id: userId } });
    group.map((item) => {
        socket.join('group_' + item.group_id);
    })
    !onlineUsers.some((user) => user.userId === userId) &&
        onlineUsers.push({
            userId,
            socketId: socket.id,
        });
    io.emit("getOnlineUsers", onlineUsers);

    socket.on("get_user_messages", async (req, callback) => {
        const { userId, chatUserId, offset } = req

        try {
            const messages = await Message.findAll({
                where: {
                    [Op.or]: [{ fromUserId: userId, toUserId: chatUserId, deleted_for_sender: 0 }, { toUserId: userId, fromUserId: chatUserId, deleted_for_recipient: 0 }],
                    isDeletedForEveryone: 0
                },
                include: [{
                    model: Message,
                    include:[{
                        model: User,
                        attributes: ['username'], // Select only the username attribute
                    }]
                }],
                limit: 20,
                offset:offset,
                order: [['createdAt', 'DESC']],
            });
            callback({
                status: 200,
                messages: messages
            });
        } catch (error) {
            callback({
                status: 500,
                messages: error.message
            });
        }
    });

    socket.on("get_group_messages", async (req, callback) => {

        const { chatUserId,offset } = req
        try {
            const messages = await Message.findAll({
                where: {
                    group_id: chatUserId,
                    isDeletedForEveryone: 0
                },
                include: [{
                    model: User,
                    attributes: ['username'], // Select only the username attribute
                },{
                    model: Message,
                    include:[{
                        model: User,
                        attributes: ['username'], // Select only the username attribute
                    }]
                }],
                limit: 20,
                offset:offset,
                order: [['createdAt', 'DESC']],
            });
            const flaggedMessages = [];
            let lastDate = null;

            messages.forEach((message) => {
                const messageDate = new Date(message.createdAt).toDateString();
                if (messageDate !== lastDate) {
                    message.dataValues.first_message_of_day = true; // Add the flag to the message
                    lastDate = messageDate;
                } else {
                    message.dataValues.first_message_of_day = false; // No flag for other messages
                }
                flaggedMessages.push(message);
            });
            callback({
                status: 200,
                messages: messages
            });
        } catch (error) {
            callback({
                status: 500,
                messages: error.message
            });
        }

    });

    socket.on("send_message", async (req, callback) => {
        const decryptedMessages =  decryptText(req);
        const messageObject = JSON.parse(decryptedMessages);
        const { toUserId, text, fromUserId, file_name, file_path, file_size, file_type, type,replay_message_id } = messageObject
        
        try {
            const date = new Date()
            date.setHours(date.getHours() + 5); // Add 5 hours for IST
            date.setMinutes(date.getMinutes() + 30);
            const encryptedText = encryptText(text, key);

            if (type == "single") {

                const message = await Message.create({
                    fromUserId: fromUserId,
                    file_url: file_path,
                    file_name: file_name,
                    file_type: file_type,
                    file_size: file_size,
                    replay_message_id:replay_message_id,
                    toUserId: toUserId,
                    text: encryptedText,
                    createAt: date.toISOString().split('T')[0]
                });
                let replyMessage = null;
                if (replay_message_id) {
                    replyMessage = await Message.findOne({ where: { id: replay_message_id },include: [{
                        model: User,
                        attributes: ['username'], // Select only the username attribute
                    }] });
                }
                const messageData = {
                    id: message.id,
                    toUserId: parseInt(message.toUserId),
                    fromUserId: message.fromUserId,
                    text: encryptedText,
                    file_url: message.file_url ?? null,
                    file_name: message.file_name,
                    file_type: message.file_type,
                    message:replyMessage,
                    deleted_for_sender: 0,
                    deleted_for_recipient: 0,
                    status: 'sent',
                    createdAt: message.createdAt,
                    updatedAt: message.updatedAt,
                }
                callback({
                    message: messageData
                });
                socket.to((toUserId).toString()).emit("new_message", messageData)
            }
            else {
                const message = await Message.create({
                    fromUserId: fromUserId,
                    file_url: file_path,
                    file_name: file_name,
                    file_type: file_type,
                    file_size: file_size,
                    replay_message_id:replay_message_id,
                    group_id: toUserId,
                    text: encryptedText,
                    createAt: date.toISOString().split('T')[0]
                });

                const fromUser = await User.findOne({ where: { id: fromUserId } });
                let replyMessage = null;
                if (replay_message_id) {
                    replyMessage = await Message.findOne({ where: { id: replay_message_id },include: [{
                        model: User,
                        attributes: ['username'], // Select only the username attribute
                    }] });
                }
                const messageData = {
                    id: message.id,
                    group_id: parseInt(message.group_id),
                    fromUserId: message.fromUserId,
                    text: encryptedText,
                    file_url: message.file_url ?? null,
                    file_name: message.file_name,
                    file_type: message.file_type,
                    message:replyMessage,
                    deleted_for_sender: 0,
                    deleted_for_recipient: 0,
                    user: { username: fromUser.username },
                    status: 'sent',
                    createdAt: message.createdAt,
                    updatedAt: message.updatedAt,
                }
                callback({
                    message: messageData
                });
                socket.to('group_' + message.group_id).emit("new_message", messageData)
            }
        } catch (error) {
            console.log('new message', error);
        }
    });


    socket.on("getAllMessages", (res) => {
        socket.to((res.fromUserId).toString()).emit("getRefreshMessage", res)
    })

    socket.on("messages-status-update", async (res) => {
        const date = new Date()
        date.setHours(date.getHours() + 5); // Add 5 hours for IST
        date.setMinutes(date.getMinutes() + 30);
        const newDate = date.toISOString().replace('T', ' ').replace('Z', '');
        try {
      
            if(res.type=='group')
            {
                if (res.status == 'read') {
                    if (res.id) {
                        const recipient = await MessageRecipient.findOne({where:{user_id:res.user_id,message_id:res.id,}});
                        if (recipient) {
                            try {
                                await MessageRecipient.update(
                                    { status: res.status, read_at: newDate },
                                    { where: { id: recipient.id}}
                                );
                            } catch (error) {
                                console.log('messages-status-update',error);
                            }
                        }else
                        {
                            try {
                                await MessageRecipient.create({status: res.status,user_id:res.user_id,message_id:res.id,group_id:res.group_id,delivered_at:newDate,read_at:newDate})
                            } catch (error) {
                                console.log('messages-status-update',error);
                            }
                        }
                    }
                    else
                    {
                        try {
                            await MessageRecipient.update(
                                { status: res.status, read_at: newDate },
                                {
                                    where: { read_at: null,group_id:res.group_id, user_id: res.user_id }
                                }
                            );
                        } catch (error) {
                            console.log('messages-status-update',error);
                        }
                    }
                }
                else
                {
                    try {
                        await MessageRecipient.create({status: res.status,user_id:res.user_id,message_id:res.id,group_id:res.group_id,delivered_at:newDate})
                    } catch (error) {
                        console.log('messages-status-update',error);
                    }
                }
            }
            else if (res.toUserId) {
                if (res.status == 'read') {
                    await Message.update(
                        { status: res.status, readAt: newDate },
                        {
                            where: { status: { [Op.ne]: 'read' }, fromUserId: res.fromUserId, toUserId: res.toUserId }
                        }
                    );
                    res.readAt=newDate;
                    res.deliveredAt=newDate;
                }
                else {
                    await Message.update(
                        { status: res.status, deliveredAt: newDate },
                        {
                            where: { status: 'sent', toUserId: res.toUserId }
                        }
                    );
                    res.deliveredAt=newDate;
                }
                if (res.fromUserId) {
                    io.to((res.fromUserId).toString()).emit("messages-status-update", res);
                }
            }
        } catch (error) {
            console.log('messages-status-update',error);  
        }
    })

    socket.on("messages_status_update_on_login", async (res) => {
        const date = new Date()
        date.setHours(date.getHours() + 5); // Add 5 hours for IST
        date.setMinutes(date.getMinutes() + 30);
        const newDate = date.toISOString().replace('T', ' ').replace('Z', '');
        try {
            
            if(res.type=='group')
            {
                const groupMessageRecipient = await MessageRecipient.findAll({where:{group_id:res.group_id,user_id:res.user_id}})
                const groupMessageIds = groupMessageRecipient.map((mr=>mr.message_id));

                const groupMessage = await Message.findAll({where:{id: { [Op.notIn]: groupMessageIds},group_id:res.group_id,fromUserId:{[Op.ne]: res.user_id}}})

                Promise.all(groupMessage.map(async (item)=>{
                    try {
                        await MessageRecipient.create({status: res.status,user_id:res.user_id,message_id:item.id,group_id:res.group_id,delivered_at:newDate})
                    } catch (error) {
                        console.log('messages_status_update_on_login',error);
                    }
                }))
            }
            else if (res.toUserId) {
                try {
                    await Message.update(
                        { status: res.status, deliveredAt: newDate },
                        {
                            where: { status: 'sent', toUserId: res.toUserId,fromUserId: res.fromUserId }
                        }
                    );
                } catch (error) {
                    console.log('messages_status_update_on_login',error);
                }
                if (res.fromUserId) {
                    io.to((res.fromUserId).toString()).emit("messages-status-update", res);
                }
            }
        } catch (error) {
            console.log('messages-status-update',error);  
        }
    })

    socket.on("delete_message", async (res) => {

        if (res.isFor == 'me') {
            if (res.type=='group') {
                await Message.update(
                    {
                        deleted_for_sender: sequelize.literal(`CASE WHEN fromUserId = ${res.fromUserId} THEN 1 ELSE deleted_for_sender END`),
                        deleted_for_recipient: sequelize.literal(`CASE WHEN toUserId = ${res.fromUserId} THEN 1 ELSE deleted_for_recipient END`)
                    },
                    {
                        where: { id: res.id }
                    }
                );
            }
            else
            {
                await Message.update(
                    {
                        deleted_for_sender: sequelize.literal(`CASE WHEN fromUserId = ${res.fromUserId} THEN 1 ELSE deleted_for_sender END`),
                        deleted_for_recipient: sequelize.literal(`CASE WHEN toUserId = ${res.fromUserId} THEN 1 ELSE deleted_for_recipient END`)
                    },
                    {
                        where: { id: res.id }
                    }
                );
            }
        }
        else if (res.isFor == 'everyone') {
            await Message.update(
                { isDeletedForEveryOne: 1 },
                {
                    where: { id: res.id }
                }
            );
            if (res.toUserId) {
                if (res.type=='group') {
                    socket.to('group_'+res.toUserId).emit("delete_message", res);
                }
                else
                {
                    socket.to((res.toUserId).toString()).emit("delete_message", res);
                }
            }
        }
    })

    socket.on('delete_all_Messages', async (res, callback) => {
        await Message.update(
            { isDeletedForEveryOne: 1 },
            {
                where: { [Op.or]: [{ fromUserId: res.toUserId, toUserId: res.fromUserId }, { toUserId: res.toUserId, fromUserId: res.fromUserId }] }
            }
        );
        socket.to((res.toUserId).toString()).emit("delete_all_Messages", res);
        callback(res);
    });

    socket.on('clear_history', async (res, callback) => {
        if (res.isFor === 'me') {
            await Message.update(
                {
                    deleted_for_sender: sequelize.literal(`CASE WHEN fromUserId = ${res.fromUserId} THEN 1 ELSE deleted_for_sender END`),
                    deleted_for_recipient: sequelize.literal(`CASE WHEN toUserId = ${res.fromUserId} THEN 1 ELSE deleted_for_recipient END`)
                },
                {
                    where: { [Op.or]: [{ fromUserId: res.toUserId, toUserId: res.fromUserId }, { toUserId: res.toUserId, fromUserId: res.fromUserId }] }
                }
            );
        }
        else if (res.isFor === 'everyone') {
            await Message.update(
                { deleted_for_sender: 1, deleted_for_recipient: 1 },
                {
                    where: { [Op.or]: [{ fromUserId: res.toUserId, toUserId: res.fromUserId }, { toUserId: res.toUserId, fromUserId: res.fromUserId }] }
                }
            );
        }
        socket.to((res.toUserId).toString()).emit("clear_history", res);
        callback(res);
    });

    socket.on('get_group_members', async (res, callback) => {
        const groupMember = await GroupMember.findAll({where:{group_id:res.group_id}, include: [{
            model: User,
            attributes: ['username','image'], // Select only the username attribute
        }]});
        callback(groupMember);
    });
    
    socket.on('search_user_by_username', async (res, callback) => {
        
        const user = await User.findOne({ 
            where: { 
                username: res.username,
            },
            attributes: ['id', 'username','image'],
        });
        callback(user);
    });

    socket.on('search_user_by_username_for_group', async (res, callback) => {
        
        const groupMember = await GroupMember.findAll({where:{group_id:res.group_id}});
        let notInIds = groupMember.map((item)=>item.user_id);

        const user = await User.findOne({ 
            where: { 
                username: res.username,
                id: { [Op.notIn]: [...notInIds,...res.selectedIds]}
            },
            attributes: ['id', 'username','image'],
        });
        callback(user);
    });

    socket.on('remove_group_members', async (res, callback) => {
       
        await Promise.all(res.removeIds.map(async (user)=>{
            await GroupMember.destroy({ where:{group_id: res.group_id, user_id: user.user_id }});
            socket.to((user.user_id).toString()).emit("add_remove_group_member", {group_id:res.group_id,type:"removed"});
            const fromUser = await User.findOne({where:{id:res.added_by_user_id}});
            const toUser = await User.findOne({where:{id:user.user_id}});
    
            const text = encryptText(fromUser.username +' removed '+toUser.username,key)
            const message = await Message.create({fromUserId:res.added_by_user_id,text,type:'notification',group_id:res.group_id});
            const messageData = {
                id: message.id,
                group_id: parseInt(message.group_id),
                fromUserId: message.fromUserId,
                text: text,
                file_url: message.file_url ?? null,
                file_name: message.file_name,
                file_type: message.file_type,
                type:"notification",
                deleted_for_sender: 0,
                deleted_for_recipient: 0,
                user: { username: fromUser.username },
                status: 'sent',
                createdAt: message.createdAt,
                updatedAt: message.updatedAt,
            }
            
            socket.to('group_' + message.group_id).emit("new_message", messageData)
        }))
        const groupMember = await GroupMember.findAll({where:{group_id:res.group_id}, include: [{
            model: User,
            attributes: ['username','image'], // Select only the username attribute
        }]});
        callback(groupMember);
    });

    socket.on('create_group', async (res, callback) => {
        try {
            console.log('create_group',res);
            const group = await Group.create({
                image: res.image,
                name: res.group_name,
                user_id: res.user_id,
                description: res.group_description,
            })
            await GroupMember.create({
                group_id: group.id,
                user_id: group.user_id,
                role:'owner'
            });
            socket.join('group_' + group.id);
            callback(group);
        } catch (error) {
            console.error('createGroup',error);
            callback(error);
        }
    });

    socket.on('add_group_members', async (res, callback) => {

        await Promise.all(res.selectedIds.map(async (userId)=>{
            await GroupMember.create({ group_id: res.group_id, user_id: userId });
            socket.to((userId).toString()).emit("add_remove_group_member", {group_id:res.group_id,type:"added"});

            const fromUser = await User.findOne({where:{id:res.added_by_user_id}});
            const toUser = await User.findOne({where:{id:userId}});
    
            const text = encryptText(fromUser.username +' added '+toUser.username,key);
            const message = Message.create({fromUserId:res.added_by_user_id,text,type:'notification',group_id:res.group_id});
            const messageData = {
                id: message.id,
                group_id: parseInt(message.group_id),
                fromUserId: message.fromUserId,
                text: text,
                file_url: message.file_url ?? null,
                file_name: message.file_name,
                file_type: message.file_type,
                type:"notification",
                deleted_for_sender: 0,
                deleted_for_recipient: 0,
                user: { username: fromUser.username },
                status: 'sent',
                createdAt: message.createdAt,
                updatedAt: message.updatedAt,
            }
            
            socket.to('group_' + message.group_id).emit("new_message", messageData);
        }))
        const groupMember = await GroupMember.findAll({where:{group_id:res.group_id}, include: [{
            model: User,
            attributes: ['username','image'], // Select only the username attribute
        }]});
       
        callback(groupMember);
    });

    socket.on('exit_group', async (res, callback) => {
        await GroupMember.destroy({ where:{group_id: res.group_id, user_id: res.user_id }});
        callback('sussesss')
    });
    socket.on('make_group_admin', async (res, callback) => {

        await GroupMember.update({role:"admin"},{where:{group_id:res.group_id,user_id:res.user_id}} );
        const groupMember = await GroupMember.findAll({where:{group_id:res.group_id}, include: [{
            model: User,
            attributes: ['username','image'], // Select only the username attribute
        }]});
        socket.to((res.user_id).toString()).emit('change_group_role',{group_id:res.group_id,user_id:res.user_id,role:"admin"})

        callback(groupMember);
    });
    socket.on('dismiss_group_admin', async (res, callback) => {
       
        await GroupMember.update({role:"user"},{where:{group_id:res.group_id,user_id:res.user_id}} );
        const groupMember = await GroupMember.findAll({where:{group_id:res.group_id}, include: [{
            model: User,
            attributes: ['username','image'], // Select only the username attribute
        }]});
        socket.to((res.user_id).toString()).emit('change_group_role',{group_id:res.group_id,user_id:res.user_id,role:"user"})

        callback(groupMember);
    });
    socket.on('add_group_member_in_socket', async (res) => {
        console.log('add_group_member_in_socket',res);
        socket.join('group_' + res.group_id);
    })
    socket.on('remove_group_member_from_socket', async (res) => {
        console.log('remove_group_member_from_socket',res);
        socket.leave('group_' + res.group_id);
    })
    socket.on('get_message_reciept', async (res, callback) => {

        const readRecipient = await MessageRecipient.findAll({where:{message_id:res.id,status:'read'}, include: [{
            as:'user',
            model: User,
            attributes: ['username','image'], // Select only the username attribute
        }]});

        const deliveredRecipient = await MessageRecipient.findAll({where:{message_id:res.id,status:'delivered'}, include: [{
            model: User,
            as:'user',
            attributes: ['username','image'], // Select only the username attribute
        }]});

        callback({messageDeliveredRecipients:deliveredRecipient,messageReadRecipients:readRecipient});
    });


    //Disconnects
    socket.on("disconnect", () => {
        onlineUsers = onlineUsers?.filter(user => user.socketId !== socket.id)
        io.emit("getOnlineUsers", onlineUsers);
        if (activeSessions[userId] === socket.id) {
            delete activeSessions[userId];
        }
    })

});

server.listen(PORT, () => {
    console.log(`Server is running on ${PORT}`);
});