const { Op } = require("sequelize");
const messageModel = require("../models/Message");
const CryptoJS = require("crypto-js");
const settingModel = require("../models/Setting");
const UserModel = require("../models/User");
const ApiRequestModel = require("../models/ApiRequest");


// Generate a random key for AES-256 encryption
const key = "mySecretKey123"; // 32 bytes = 256 bits

// Encrypt function
const encryptText = (text, key) => {
    const encrypted = CryptoJS.AES.encrypt(text, key).toString();
    return encrypted;
}

const sendMessage = async (req, res) => {

    const { username, text } = req.body
    
    if (!username) {
        return res.status(500).json({error:"username Field is required"});
    }
    if (!text && !req.files) {
        return res.status(500).json({error:"text Field is required"});
    }
    const user = await UserModel.findOne({ where: { username } });

    if (!user) {
        return res.status(500).json({error:"user not found"});
    }
    try {
        const userId = user.id;

        ApiRequestModel.create({
            userId:req.user.id,
            endPoint:'/api/company/send/message',
            status:1
        })
        if (req.files) {
            
            const file = req.files.file;
            const fileSize = req.files.file.size;
            const fileExtension = file.name.slice((file.name.lastIndexOf('.') - 1 >>> 0) + 2);
            
            const randomName = generateRandomString(20);
            if (fileSize < (5 * 1024 * 1024)) {
                let uploadPath = 'upload/images/' + randomName+'.'+fileExtension;
                file.mv(uploadPath, async function (err) {
                    if (err) {
                        return res.status(500).json({
                            success: 0,
                            message: 'Error uploading file',
                            error: err.message // Include error message in the response
                        });
                    }
                    const fileName = '/images/' + randomName+'.'+fileExtension;
                    const encryptedText = encryptText(text, key);
                    const message = await messageModel.create({
                        fromUserId: req.user.id,
                        toUserId: user.id,
                        text: encryptedText,
                        file_url: fileName,
                        file_name:file.name,
                        file_size:file.size,
                        file_type:file.mimetype,
                        sentFrom:'API'
                    });

                    const messageResData = {
                        userId: message.toUserId,
                        text: text,
                        file_url: message.file_url,
                        file_size: message.file_size,
                        file_type: message.file_type,
                        status: message.status,
                        createdAt: message.createdAt,
                        updatedAt: message.updatedAt,
                    }
                    const messageData = {
                        id: message.id,
                        toUserId: parseInt(message.toUserId),
                        fromUserId: message.fromUserId,
                        text: encryptedText,
                        file_url: message.file_url ?? null,
                        file_name: message.file_name,
                        file_type: message.file_type,
                        deleted_for_sender: 0,
                        deleted_for_recipient: 0,
                        status: 'sent',
                        createdAt: message.createdAt,
                        updatedAt: message.updatedAt,
                    }
                    req.io.to((userId).toString()).emit('new_message', messageData);
                    res.status(200).json(messageResData);
                });
            }
            else
            {
                return res.status(500).json({error:"File size maximum 5 MB allow"});
            }
        } else {
            const encryptedText = encryptText(text, key);
            const message = await messageModel.create({
                fromUserId: req.user.id,
                toUserId: userId,
                text: encryptedText,
                sentFrom:'API'
            });

            const messageResData = {
                userId: message.toUserId,
                toUserId: message.toUserId,
                fromUserId:message.fromUserId,
                text: text,
                file_url: null,
                status: message.status,
                createdAt: message.createdAt,
                updatedAt: message.updatedAt,
            }
            const messageData = {
                id: message.id,
                toUserId: parseInt(message.toUserId),
                fromUserId: message.fromUserId,
                text: encryptedText,
                file_url: message.file_url ?? null,
                file_name: message.file_name,
                file_type: message.file_type,
                deleted_for_sender: 0,
                deleted_for_recipient: 0,
                status: 'sent',
                createdAt: message.createdAt,
                updatedAt: message.updatedAt,
            }
            req.io.to((userId).toString()).emit('new_message', messageData);
            res.status(200).json(messageResData);
        }

    } catch (error) {
        res.status(500).json('error' + error)
    }
}

const getMessages = async (req, res) => {

    const { chatId } = req.params
    try {

        const messages = await messageModel.findAll({
            where: {
                [Op.or]: [{ fromUserId: req.userId, toUserId: chatId }, { toUserId: req.userId, fromUserId: chatId }],
            },
        });

        res.status(200).json(messages);

    } catch (error) {
        res.status(500).json(error.message);
    }
}

const getMediaMessages = async (req, res) => {

    const { chatId } = req.params
    try {

        const messages = await messageModel.findAll({
            where: {
                [Op.or]: [{ fromUserId: req.userId, toUserId: chatId }, { toUserId: req.userId, fromUserId: chatId }],
                file_url: {
                    [Op.ne]: null
                }
            },
        });

        res.status(200).json(messages);

    } catch (error) {
        res.status(500).json(error.message);
    }
}

const updateMessagesStatus = async (req, res) => {
    try {
        // Parse array of message IDs from request body
        const { messageIds } = req.body;
        const { status } = req.body;

        // Update status of messages with IDs in the array
        const updatedRows = await messageModel.update(
            { status },
            { where: { id: messageIds } }
        );

        // Check if any rows were affected by the update operation
        if (updatedRows[0] === 0) {
            // If no rows were affected, none of the message IDs were found
            return res.status(404).json({ error: "No messages found with the provided IDs" });
        }

        // Return success response
        res.status(200).json({ success: true });
    } catch (error) {
        res.status(500).json({ error: error.message });
    }
};


const updateMessageStatusByChatId = async (req, res) => {
    const { chatId } = req.params;
    const { status } = req.body;

    try {

        const updatedRows = await messageModel.update(
            { status },
            { where: { chatId: chatId } }
        );


        // Check if any rows were affected by the update operation
        if (updatedRows[0] === 0) {
            // If no rows were affected, the message with the given ID was not found
            return res.status(404).json({ error: "Message not found" });
        }

        // Return success response
        res.status(200).json({ success: true });
    } catch (error) {
        res.status(500).json({ error: error.message });
    }
}

const updateStatusAllMessages = async (req, res) => {
    const { chatId, senderId } = req.params;

    try {
        // Update status of all messages where chatId and senderId match
        const updatedMessages = await messageModel.update(
            { status: "read" }, // Update status to "read"
            { where: { chatId, senderId } } // Filter by chatId and senderId
        );

        res.status(200).json({ message: "Status updated for all matching messages" });
    } catch (error) {
        res.status(500).json({ error: error.message });
    }
}

const updateStatusMessages = async (req, res) => {
    const { toUserId } = req.params;
    const { status } = req.body;

    try {
        // Update status of all messages where chatId and senderId match
        const updatedMessages = await messageModel.update(
            { status: status }, // Update status to "read"
            { where: { fromUserId:toUserId,toUserId:req.userId} } // Filter by chatId and senderId
        );

        res.status(200).json({ message: "Status updated for all matching messages" });
    } catch (error) {
        res.status(500).json({ error: error.message });
    }
}
const deleteMessage = async (req, res) => {

    try {

        const messageId = req.params.messageId;

        const deletedMessage = await messageModel.destroy({ where: { id: messageId } });

        if (deletedMessage === 0) {
            return res.status(400).json({ error: 'message not found' });
        }

        return res.status(200).json({ message: 'Message deleted successfully...' });

    } catch (error) {
        return res.status(500).json({ error: 'Internal server error', error })
    }
}

const deleteMessageByChatId = async (req, res) => {

    try {

        const chatId = req.params.chatId;

        const deletedMessageByChatId = await messageModel.destroy({ where: { chatId: chatId } });

        if (deletedMessageByChatId === 0) {
            return res.status(400).json({ error: 'message not found' });
        }

        return res.status(200).json({ message: 'Message deleted successfully...' })

    } catch (error) {
        return res.status(500).json({ error: 'Internal server error', error })
    }
}

const deleteMessagesByChatIds = async (req, res) => {
    try {
        const { chatIds } = req.body;

        // Delete chats with the specified chatIds
        const deletedMessagesCount = await messageModel.destroy({ where: { chatId: chatIds } });

        // Check if any chats were deleted
        if (deletedMessagesCount === 0) {
            return res.status(404).json({ error: 'No chats found with the provided chatIds' });
        }

        return res.status(200).json({ message: 'messages deleted successfully' });
    } catch (error) {
        console.error('Error deleting messages:', error);
        return res.status(500).json({ error: 'Internal server error', error });
    }
};

const sendBroadcastMessage = async (req, res) => {
    const toSend = req.params.user;

    const { text } = req.body
    try {
        if (req.files) {
            
            const file = req.files.file;
            const fileSize = req.files.file.size;
            const fileExtension = file.name.slice((file.name.lastIndexOf('.') - 1 >>> 0) + 2);
            
            const randomName = generateRandomString(20);
            if (fileSize < (5 * 1024 * 1024)) {
                let uploadPath = 'upload/images/' + randomName+'.'+fileExtension;
                file.mv(uploadPath, async function (err) {
                    if (err) {
                        return res.status(500).json({
                            success: 0,
                            message: 'Error uploading file',
                            error: err.message // Include error message in the response
                        });
                    }
                    const fileName = '/images/' + randomName+'.'+fileExtension;
                    const encryptedText = encryptText(text, key);
                    let users=[];
                    if (toSend == 'company') {
                        users = await UserModel.findAll({where:{role:2}});
                    }
                    else
                    {
                        users = await UserModel.findAll({where:{role:3}});
                    }
                    users.map(async (user)=>{
                        const message = await messageModel.create({
                            fromUserId: req.user.id,
                            toUserId: user.id,
                            text: encryptedText,
                            file_url: fileName,
                            file_name:file.name
                        });

                        const messageData = {
                            userId: message.toUserId,
                            text: text,
                            file_url: message.file_url,
                            status: message.status,
                            createdAt: message.createdAt,
                            updatedAt: message.updatedAt,
                        }
                        const onlineUsers = req.onlineUsers;
                        const socketUser = onlineUsers.find(user => user.userId == user.id);
                        if (socketUser) {
                            req.io.to(socketUser.socketId).emit('getRefreshMessage', messageData);
                        }
                    })
                    res.status(200).json({'status':1,'msg':'success'});
                });
            }
        } else {
            const encryptedText = encryptText(text, key);
            let users=[];
            if (toSend == 'company') {
                users = await UserModel.findAll({where:{role:2}});
            }
            else
            {
                users = await UserModel.findAll({where:{role:3}});
            }
            users.map(async (user)=>{
                const message = await messageModel.create({
                    fromUserId: req.user.id,
                    toUserId:  user.id,
                    text: encryptedText,
                });

                const messageData = {
                    userId: message.toUserId,
                    text: text,
                    file_url: null,
                    status: message.status,
                    createdAt: message.createdAt,
                    updatedAt: message.updatedAt,
                }

                const onlineUsers = req.onlineUsers;
                const socketUser = onlineUsers.find(user => user.userId ==  user.id);
                if (socketUser) {
                    req.io.to(socketUser.socketId).emit('getRefreshMessage', messageData);
                }
            });
            res.status(200).json({'status':1,'msg':'success'});
        }

    } catch (error) {
        res.status(500).json('error' + error)
    }
}
const getSettings = async (req, res) =>{
    try {
        let result = {};
        const settings = await settingModel.findAll();
        settings.map((set)=> {result[set.key]=set.value} );

        res.status(200).json(result);
    } catch (error) {
        console.error('Error editing profile:', error);
        res.status(500).json({ error: error});
    }
};
const saveSettings = async (req, res) =>{
    try {
        const setting = req.body;
        const settingData = Object.entries(setting).map(([key, value]) => ({
            key,
            value
        }));
        await settingModel.bulkCreate(settingData, { 
            updateOnDuplicate: ['value'] // Specify which columns to update on duplicate key
        });
       
        // // Update the user's profile in the database with the provided fields
        
        // await settingModel.update(
        //     setting,
        //     { where: { id: userId } }
        // );

     
        res.status(200).json({ success: true, message: 'Setting updated successfully' });
    } catch (error) {
        console.error('Error editing profile:', error);
        res.status(500).json({ error: error});
    }
};
const generateRandomString = (length) => {
    const characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    let result = '';
    for (let i = 0; i < length; i++) {
      result += characters.charAt(Math.floor(Math.random() * characters.length));
    }
    return result;
};
module.exports = {
    sendMessage,
    getMessages,
    updateStatusAllMessages,
    deleteMessage,
    deleteMessageByChatId,
    deleteMessagesByChatIds,
    updateMessageStatusByChatId,
    updateMessagesStatus,
    updateStatusMessages,
    getMediaMessages,
    sendBroadcastMessage,
    getSettings,
    saveSettings
}