const UserModel = require("../models/User");
const Joi = require('joi');
const bcrypt = require('bcrypt');
const jwt = require("jsonwebtoken");
const { Op } = require("sequelize");
const { sequelize, Sequelize } = require('../connection'); // assuming you have a connection file
const { QueryTypes } = require('sequelize');
const messageModel = require("../models/Message");
const ApiRequestModel= require("../models/ApiRequest");
const settingModel = require("../models/Setting");
const MessageRestoreRequest = require("../models/MessageRestoreRequest");

const CryptoJS = require("crypto-js");
const User = require("../models/User");
const Group = require("../models/Group");
const GroupMember = require("../models/GroupMembers");


// Generate a random key for AES-256 encryption
const key = "mySecretKey123"; // 32 bytes = 256 bits

// Encrypt function
const encryptText = (text, key) => {
    const encrypted = CryptoJS.AES.encrypt(text, key).toString();
    return encrypted;
}


// validations
const schema = Joi.object({
    firstName: Joi.string().min(3).max(30).required(),
    lastName: Joi.string().min(3).max(30).required(),
    username: Joi.string().min(3).max(30).required(),
    email: Joi.string().email().required(),
    image: Joi.string(),
    mobileNumber: Joi.string().pattern(new RegExp(/^[6789]\d{9}$/)).required(),
    password: Joi.string().pattern(new RegExp('^[a-zA-Z0-9]{3,30}$')).required(),
    subscriptionExpiryDate: Joi.string(),
    role: Joi.number(),
});


// register api
const register = async (req, res) => {
    try {
        const { error, value } = schema.validate(req.body);
        if (error) {
            console.log(error);
            return res.status(400).json({ error: error.details[0].message });
        }

        const { firstName, lastName, username, email, mobileNumber, password,subscriptionExpiryDate,image,role } = req.body;

        let existUserName = await UserModel.findOne({ where: { username: req.body.username } });
        let existUserEmail = await UserModel.findOne({ where: { email: req.body.email } });

        if (existUserName) {
            return res.status(400).json({ error:"Username already exists..."});
        }

        if (existUserEmail) {
            return res.status(400).json({ error:"Email already exists..."});
        }
        const payload = {
            username,
        };
        let user;
        if (role===3) {
            user = new UserModel({
                firstName,
                lastName,
                username,
                image,
                email,
                mobileNumber,
                password,
                role:3,
            });   
        }
        else
        {
            user = new UserModel({
                firstName,
                lastName,
                username,
                image,
                email,
                mobileNumber,
                password,
                role,
                subscriptionExpiryDate
            });  
            
            user.apiToken = jwt.sign(payload, process.env.JWT_SECRET_KEY, { expiresIn: '300d' });
        }
        
        
        const salt = await bcrypt.genSalt(10);
        user.password = await bcrypt.hash(user.password, salt);
        user.refreshToken = jwt.sign(payload, process.env.JWT_SECRET_KEY, { expiresIn: '30d' });

        await user.save();
        res.status(200).json({ success: "Company Created successfully" });

    } catch (error) {
        console.log(error);
        res.status(500).json({ error: error })
    }

}


const findCompany = async (req, res) => {
    const id = req.params.id; // Assuming the parameter is in the URL
    try {
        const user = await UserModel.findOne({ where: { id } });
        if (!user) {
            return res.status(404).json({ error: "User not found" });
        }

        res.status(200).json(user);

    } catch (error) {
        res.status(500).json({ error: error.message });
    }
}

const searchUser = async (req, res) => {

    try {
        const { username } = req.body;

        // Check if userName is provided
        if (!username) {
            return res.status(400).json({ error: 'userName parameter is required' });
        }

        // Find users with matching userName
        const users = await UserModel.findOne({where: {username}});

        // Customize the user data as needed
        // const userArray = users.map(user => ({
        //     id: user.id,
        //     userName: user.userName,
        //     email: user.email,
        //     mobileNumber: user.mobileNumber,
        //     role: user.role,
        //     // type: user.userType,
        // }));

        res.status(200).json(users);

    } catch (error) {
        res.status(500).json({ error: error.message });
    }
}

const getAllUsers = async (req, res) => {
    try {
        const users = await UserModel.findAll({where:{role:3}});
        res.status(200).json(users);

    } catch (error) {
        res.status(500).json({ error: error.message });
    }
}

const getAllCompanies = async (req, res) => {
    try {
        const users = await UserModel.findAll({where:{role:2}});

        res.status(200).json(users);

    } catch (error) {
        res.status(500).json({ error: error.message });
    }
}


const getApiRequests = async (req, res) => {

    const { apiUserId, startDate, endDate } = req.body;
    try {
        let query ={userId:apiUserId};
        if (endDate!==0) {
            query ={userId:apiUserId,
                createdAt: {
                [Op.gte]: startDate, // Greater than or equal to startDate
                [Op.lte]: endDate     // Less than or equal to endDate
            }};
        }
        const request  = await ApiRequestModel.findAll({
            where:query
        });

        res.status(200).json(request);
    } catch (error) {
        res.status(500).json({ error: error.message });
    }
}


// Function to handle profile image upload
const uploadProfileImage = async (req, res) => {
    try {

        // Get the user ID from the request
        const userId = req.params.userId;
        const { imageUrl } = req.body

        // Update the user's profile image path in the database
        await UserModel.update({ image: imageUrl }, { where: { id: userId } });

        // Send a success response
        res.status(200).json({ message: 'Profile image uploaded successfully' });
    } catch (error) {
        console.error('Error uploading profile image:', error);
        res.status(500).json({ error: 'Internal server error', error });
    }
};


const updateProfile = async (req, res) => {
    try {
        // Get the user ID from the request
        const userId = req.params.id;

        // Get the fields to update from the request body

        const { firstName,username,lastName,email,mobileNumber,password,image ,subscriptionExpiryDate,status,role} = req.body;

        const user = await UserModel.findOne({ where: { username } });

        const payload = {
            username,
        };
        let apiToken
        if (role==2 && (user.apiToken=='' || user.apiToken==null)) {
            apiToken = jwt.sign(payload, process.env.JWT_SECRET_KEY, { expiresIn: '365d' });
        }

        const fieldsToUpdate = {
            firstName,
            lastName,
            email,
            mobileNumber,
            image,
            status,
            role,
            apiToken,
            subscriptionExpiryDate
        };


        if (password!=null && password!='') {           
            const salt = await bcrypt.genSalt(10);
            fieldsToUpdate.password =  await bcrypt.hash(password, salt);
        }
        // Update the user's profile in the database with the provided fields
        const [updatedRowCount] = await UserModel.update(
            fieldsToUpdate,
            { where: { id: userId } }
        );

        // Check if the user was found and updated
        // if (updatedRowCount === 0) {
        //     return res.status(404).json({ error: 'User not found' });
        // }
        // const user = await UserModel.findOne({ where: { id:userId } });
        // const token = createToken(user.id,user.username,user.email,user.role,user.firstName+' '+user.lastName);

        // const userData = {
        //     id: user.id,
        //     firstName: user.firstName, 
        //     lastName: user.lastName, 
        //     username: user.username, 
        //     email: user.email,
        //     mobileNumber: user.mobileNumber,
        //     type: user.userType,
        //     image: user.image,
        //     refreshToken: user.refreshToken,
        //     token,
        //     role: user.role,
        // };
        // Send a success response
        res.status(200).json({ success: true, message: 'Profile updated successfully' });
    } catch (error) {
        console.error('Error editing profile:', error);
        res.status(500).json({ error: error});
    }
}

const deleteUserProfile = async (req, res) => {

    try {

        // Get the user ID from the request
        const userId = req.params.userId;
        console.log(userId);

        // Find the user by ID and delete it
        const deletedUser = await UserModel.destroy({ where: { id: userId } });

        // Check if the user was found and deleted
        if (deletedUser === 0) {
            return res.status(404).json({ error: 'User not found' });
        }

        // Send a success response
        res.status(200).json({ success: 'profile deleted successfully' });

    } catch (error) {
        return res.status(500).json({ error: 'Internak server error', error });
    }
}

const getChatUserLists = async (req, res) => {
    const userId = req.params.userId;
    try {
        const chatUserIds = await sequelize.query(`SELECT DISTINCT userId,text
            FROM (
                SELECT fromUserId AS userId,text FROM messages WHERE toUserId = ${userId} AND isDeletedForEveryone=0 AND group_id=0
                UNION
                SELECT toUserId AS userId,text FROM messages WHERE fromUserId = ${userId} AND isDeletedForEveryone=0 AND group_id=0
            ) AS users`
            , {
                type: QueryTypes.SELECT,
            });
        const groupList = await sequelize.query(`SELECT groups.* FROM groups JOIN group_members on group_id=groups.id where group_members.user_id= ${userId}`,
            {
                type: QueryTypes.SELECT,
            });
        let usersList = [];
        if (chatUserIds.length != 0) {
            const userIds = chatUserIds.map(user => user.userId);
            const allChats = await sequelize.query(`SELECT t1.* FROM messages t1 JOIN ( SELECT LEAST(fromUserId, toUserid) user1, GREATEST(fromUserId, toUserId) user2, MAX(timestamps) timestamps FROM messages t2 GROUP BY user1, user2 ) t3 ON t1.fromUserid IN (t3.user1, t3.user2) AND t1.toUserId IN (t3.user1, t3.user2) AND t1.timestamps = t3.timestamps where (fromUserId=${userId} or toUserId=${userId}) and isDeletedForEveryone=0 ;`,
                {
                    type: QueryTypes.SELECT,
                });
            usersList = await sequelize.query(`SELECT id,firstName,lastName,username as name, username,image,role FROM users where id in(${userIds})`,
                {
                    type: QueryTypes.SELECT,
                });
            allChats.map((user) => {
                return user.userId = user.fromUserId !== userId ? user.fromUserId : user.toUserId
            })
            await Promise.all(usersList.map(async (user) => {
                const lastMessage = allChats.find((msg) => msg.userId === user.id);
                if (lastMessage != undefined) {
                    if (lastMessage.deleted_for_sender === 0 && lastMessage.deleted_for_recipient === 0)
                        user.message = lastMessage
                }
            }))
        }
        console.log('usersList',usersList);
        if (groupList.length != 0) {
            await Promise.all(groupList.map(async (group) => {
                const lastMessage = await sequelize.query(`SELECT * FROM messages where group_id=${group.id} and isDeletedForEveryone=0 order by id desc;`,
                    {
                        type: QueryTypes.SELECT,
                    });
                if (lastMessage.length != 0) {
                    if (lastMessage[0].deleted_for_sender === 0 && lastMessage[0].deleted_for_recipient === 0)
                        return group.message = lastMessage[0]
                }
            }))
        }

        usersList = usersList.concat(groupList)

        if (usersList.length != 0) {
            usersList.sort((a, b) => {
                const createdAtA = a.message ? a.message.timestamps : 0; // If a message exists, get its createdAt timestamp; otherwise, use 0
                const createdAtB = b.message ? b.message.timestamps : 0; // If a message exists, get its createdAt timestamp; otherwise, use 0
                return createdAtB - createdAtA; // Sort in descending order
            });
            res.status(200).json(usersList);
        }
        else {
            res.status(200).json([]);
        }

    } catch (error) {
        res.status(500).json('error' + error);
    }
}
const getMessages = async (req, res) => {

    const { userId,toId } = req.params
    try {

        const messages = await messageModel.findAll({
            where: {
                [Op.or]: [{ fromUserId: userId, toUserId: toId }, { toUserId: userId, fromUserId: toId }],
            },
            limit: 20,
            order: [['createdAt', 'DESC']],
        });

        res.status(200).json(messages);

    } catch (error) {
        res.status(500).json(error.message);
    }
}

const getGroupMessages = async (req, res) => {

    const { userId,chatUserId } = req.params
    try {

        const messages = await messageModel.findAll({
            where: {
                group_id: chatUserId,
                isDeletedForEveryone: 0
            },
            include: [{
                model: User,
                attributes: ['username'], // Select only the username attribute
            }],
            limit: 20,
            order: [['createdAt', 'DESC']],
        });

        res.status(200).json(messages);

    } catch (error) {
        res.status(500).json(error.message);
    }
}

const getGroupInfo = async (req, res) => {

    const { userId,groupId } = req.params
    try {
        const group = await Group.findOne({
            where: {
                id: groupId,
            },
        });
        const groupMembers = await GroupMember.findAll({where:{group_id:groupId}, include: [{
            model: User,
            attributes: ['username','image'], // Select only the username attribute
        }]});
        res.status(200).json({group,groupMembers});

    } catch (error) {
        res.status(500).json(error.message);
    }
}

const messageRestore = async (req, res) => {

    const { userId,toId } = req.params
    const { startDate,endDate,requestId } = req.body
    try {
        
        await messageModel.update(
            { isDeletedForEveryOne:0,deleted_for_sender:0,deleted_for_recipient:0},
            { 
                where: { 
                    [Op.or]: [{ fromUserId: userId, toUserId: toId }, { toUserId: userId, fromUserId: toId }],
                    createdAt:{
                        [Op.between]: [startDate, endDate]
                    }
                } 
            }
        );
        await MessageRestoreRequest.update(
            {status:'approved'},
            { where: {id:requestId}}
        )

        res.status(200).json({'status':1,'msg':'success'});

    } catch (error) {
        res.status(500).json(error.message);
    }

}
const sendBroadcastMessage = async (req, res) => {
    const toSend = req.params.user;

    const { text } = req.body

    if (!text && !req.files) {
        return res.status(500).json({error:"text Field is required"});
    }

    try {
        if (req.files) {
            
            const file = req.files.file;
            const fileSize = req.files.file.size;
            const fileExtension = file.name.slice((file.name.lastIndexOf('.') - 1 >>> 0) + 2);
            
            const randomName = generateRandomString(20);
            if (fileSize < (5 * 1024 * 1024)) {
                let uploadPath = 'upload/images/' + randomName+'.'+fileExtension;
                file.mv(uploadPath, async function (err) {
                    if (err) {
                        return res.status(500).json({
                            success: 0,
                            message: 'Error uploading file',
                            error: err.message // Include error message in the response
                        });
                    }
                    const fileName = '/images/' + randomName+'.'+fileExtension;
                    const encryptedText = encryptText(text, key);
                    let users=[];
                    if (toSend == 'company') {
                        users = await UserModel.findAll({where:{role:2}});
                    }
                    else
                    {
                        users = await UserModel.findAll({where:{role:3}});
                    }
                    users.map(async (user)=>{
                        const message = await messageModel.create({
                            fromUserId: req.userId,
                            toUserId: user.id,
                            text: encryptedText,
                            file_url: fileName,
                            file_name:file.name
                        });

                        const messageData = {
                            userId: message.toUserId,
                            text: text,
                            file_url: message.file_url,
                            status: message.status,
                            createdAt: message.createdAt,
                            updatedAt: message.updatedAt,
                        }
                        const onlineUsers = req.onlineUsers;
                        const socketUser = onlineUsers.find(user => user.userId == user.id);
                        if (socketUser) {
                            req.io.to(socketUser.socketId).emit('getRefreshMessage', messageData);
                        }
                    })
                    res.status(200).json({'status':1,'msg':'success'});
                });
            }
        } else {
            const encryptedText = encryptText(text, key);
            let users=[];
            if (toSend == 'company') {
                users = await UserModel.findAll({where:{role:2}});
            }
            else
            {
                users = await UserModel.findAll({where:{role:3}});
            }
            const onlineUsers = req.onlineUsers;

            users.map(async (user)=>{
                const message = await messageModel.create({
                    fromUserId: req.userId,
                    toUserId:  user.id,
                    text: encryptedText,
                });

                const messageData = {
                    userId: message.toUserId,
                    text: text,
                    file_url: null,
                    status: message.status,
                    createdAt: message.createdAt,
                    updatedAt: message.updatedAt,
                }

               
                const socketUser = onlineUsers.find((user1)=>user1.userId ==  user.id);

                if (socketUser) {
                    req.io.to(socketUser.socketId).emit('getRefreshMessage', messageData);
                }
            });
            res.status(200).json({'status':1,'msg':'success'});
        }

    } catch (error) {
        res.status(500).json('error' + error)
    }
}

const getMessageRestoreRequests = async (req, res) => {
   

    try {
        const requests = await MessageRestoreRequest.findAll({
            include: [{
              model: UserModel,
              as: 'fromUsers',
            },
            {
                model: UserModel,
                as: 'toUsers',
              }],
            order: [
                ['id', 'DESC'],
            ],
          });
        res.status(200).json(requests);
    } catch (error) {
        return res.status(500).json({ error: 'Internal server error', error });
    }
}

const generateRandomString = (length) => {
    const characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    let result = '';
    for (let i = 0; i < length; i++) {
      result += characters.charAt(Math.floor(Math.random() * characters.length));
    }
    return result;
};

module.exports = {
    register,
    findCompany,
    getAllUsers,
    searchUser,
    uploadProfileImage,
    updateProfile,
    deleteUserProfile,
    getAllCompanies,
    getChatUserLists,
    getMessages,
    sendBroadcastMessage,
    getApiRequests,
    getMessageRestoreRequests,
    messageRestore,
    getGroupMessages,
    getGroupInfo

}

