const chatModel = require("../models/Chat");
const { sequelize, Sequelize } = require('../connection'); // assuming you have a connection file
const { QueryTypes } = require('sequelize');
const GroupMember = require("../models/GroupMembers");
const Group = require("../models/Group");

const createChat = async (req, res) => {
    const { members } = req.body;
    try {

        // Check if members is undefined or not an array
        if (!members || !Array.isArray(members)) {
            return res.status(400).json({ error: 'Invalid or missing members property in the request body' });
        }

        const membersArray = members.map(Number);

        const existingChat = await sequelize.query(
            'SELECT `id`, `members` FROM `chats` WHERE JSON_CONTAINS(`members`, :membersArray)',
            {
                replacements: { membersArray: JSON.stringify(members) },
                type: Sequelize.QueryTypes.SELECT,
                model: chatModel,
            }
        );
        if (existingChat.length > 0) {
            // If it exists, return the existing chat
            return res.status(200).json(existingChat[0]);
        }

        const newChat = await chatModel.create({
            members,
        });
        res.status(200).json(newChat);
    } catch (error) {
        res.status(500).json('error' + error);
    }
};


const getAllChats = async (req, res) => {
    const userId = req.userId;
    try {
        const chatUserIds = await sequelize.query(`SELECT DISTINCT userId,text
            FROM (
                SELECT fromUserId AS userId,text FROM messages WHERE toUserId = ${userId} AND isDeletedForEveryone=0 AND group_id=0
                UNION
                SELECT toUserId AS userId,text FROM messages WHERE fromUserId = ${userId} AND isDeletedForEveryone=0 AND group_id=0
            ) AS users`
            , {
                type: QueryTypes.SELECT,
            });
        const groupList = await sequelize.query("SELECT `groups`.*, group_members.role, `groups`.name as username FROM `groups` JOIN group_members on group_id=`groups`.id where group_members.user_id="+ userId,
            {
                type: QueryTypes.SELECT,
            });
        let usersList = [];
        if (chatUserIds.length != 0) {
            const userIds = chatUserIds.map(user => user.userId);
            const allChats = await sequelize.query(`SELECT t1.* FROM messages t1 JOIN ( SELECT LEAST(fromUserId, toUserid) user1, GREATEST(fromUserId, toUserId) user2, MAX(timestamps) timestamps FROM messages t2 GROUP BY user1, user2 ) t3 ON t1.fromUserid IN (t3.user1, t3.user2) AND t1.toUserId IN (t3.user1, t3.user2) AND t1.timestamps = t3.timestamps where (fromUserId=${userId} or toUserId=${userId}) and isDeletedForEveryone=0 ;`,
                {
                    type: QueryTypes.SELECT,
                });
            usersList = await sequelize.query(`SELECT id,firstName,lastName,username,image,role FROM users where id in(${userIds})`,
                {
                    type: QueryTypes.SELECT,
                });
            allChats.map((user) => {
                return user.userId = user.fromUserId !== userId ? user.fromUserId : user.toUserId
            })
            await Promise.all(usersList.map(async (user) => {
                user.type ="single";
                const lastMessage = allChats.find((msg) => msg.userId === user.id);
                if (lastMessage != undefined) {
                    if (lastMessage.deleted_for_sender === 0 && lastMessage.deleted_for_recipient === 0)
                        user.message = lastMessage
                }
            }))
        }
        if (groupList.length != 0) {
            await Promise.all(groupList.map(async (group) => {
                group.type ="group";
                const lastMessage = await sequelize.query(`SELECT * FROM messages where group_id=${group.id} and isDeletedForEveryone=0 and type!='notification' order by id desc;`,
                    {
                        type: QueryTypes.SELECT,
                    });
                if (lastMessage.length != 0) {
                    if (lastMessage[0].deleted_for_sender === 0 && lastMessage[0].deleted_for_recipient === 0)
                        return group.message = lastMessage[0]
                }
            }))
        }

        usersList = usersList.concat(groupList)

        if (usersList.length != 0) {
            usersList.sort((a, b) => {
                const createdAtA = a.message ? a.message.timestamps : 0; // If a message exists, get its createdAt timestamp; otherwise, use 0
                const createdAtB = b.message ? b.message.timestamps : 0; // If a message exists, get its createdAt timestamp; otherwise, use 0
                return createdAtB - createdAtA; // Sort in descending order
            });
            res.status(200).json(usersList);
        }
        else {
            res.status(200).json([]);
        }

    } catch (error) {
        res.status(500).json('error' + error);
    }
}

const getChatUserLists = async (req, res) => {
    const userId = req.userId;
    try {
        const chatUserIds = await sequelize.query(`SELECT DISTINCT userId,text
            FROM (
                SELECT fromUserId AS userId,text FROM messages WHERE toUserId = ${userId} AND isDeletedForEveryone=0
                UNION
                SELECT toUserId AS userId,text FROM messages WHERE fromUserId = ${userId} AND isDeletedForEveryone=0
            ) AS users`
            , {
                type: QueryTypes.SELECT,
            });

        if (chatUserIds.length != 0) {

            const userIds = chatUserIds.map(user => user.userId);

            const allChats = await sequelize.query(`SELECT t1.* FROM messages t1 JOIN ( SELECT LEAST(fromUserId, toUserid) user1, GREATEST(fromUserId, toUserId) user2, MAX(timestamps) timestamps FROM messages t2 GROUP BY user1, user2 ) t3 ON t1.fromUserid IN (t3.user1, t3.user2) AND t1.toUserId IN (t3.user1, t3.user2) AND t1.timestamps = t3.timestamps where (fromUserId=${userId} or toUserId=${userId}) and isDeletedForEveryone=0 ;`,
                {
                    type: QueryTypes.SELECT,
                });

            const usersList = await sequelize.query(`SELECT username,lastName,firstName,role,image,id as userId FROM users where id in(${userIds})`,
                {
                    type: QueryTypes.SELECT,
                });

            allChats.map((user) => {
                return user.userId = user.fromUserId !== userId ? user.fromUserId : user.toUserId
            })

            usersList.map((user) => {
                const lastMessage = allChats.find((msg) => msg.userId === user.userId);
                if (lastMessage != undefined) {
                    if (lastMessage.deleted_for_sender === 0 && lastMessage.deleted_for_recipient === 0)
                        return user.message = lastMessage
                }
            })

            usersList.sort((a, b) => {
                const createdAtA = a.message ? a.message.timestamps : 0; // If a message exists, get its createdAt timestamp; otherwise, use 0
                const createdAtB = b.message ? b.message.timestamps : 0; // If a message exists, get its createdAt timestamp; otherwise, use 0
                return createdAtB - createdAtA; // Sort in descending order
            });
            res.status(200).json(usersList);
        }
        else {
            res.status(200).json([]);
        }

    } catch (error) {
        res.status(500).json('error' + error);
    }
}

const getAllData = async (req, res) => {
    const userId = req.userId;
    console.log("dfdsf lkdjfkldsjfklsdj lk", userId);
    try {
        const chatUserIds = await sequelize.query(`SELECT DISTINCT userId,text
            FROM (
                SELECT fromUserId AS userId,text FROM messages WHERE toUserId = ${userId} AND isDeletedForEveryone=0
                UNION
                SELECT toUserId AS userId,text FROM messages WHERE fromUserId = ${userId} AND isDeletedForEveryone=0
            ) AS users`
            , {
                type: QueryTypes.SELECT,
            });

        if (chatUserIds.length != 0) {

            const userIds = chatUserIds.map(user => user.userId);

            const allChats = await sequelize.query(`SELECT t1.* FROM messages t1 JOIN ( SELECT LEAST(fromUserId, toUserid) user1, GREATEST(fromUserId, toUserId) user2, MAX(timestamps) timestamps FROM messages t2 GROUP BY user1, user2 ) t3 ON t1.fromUserid IN (t3.user1, t3.user2) AND t1.toUserId IN (t3.user1, t3.user2) AND t1.timestamps = t3.timestamps where (fromUserId=${userId} or toUserId=${userId}) and isDeletedForEveryone=0 ;`,
                {
                    type: QueryTypes.SELECT,
                });

            const usersList = await sequelize.query(`SELECT * FROM users where id in(${userIds})`,
                {
                    type: QueryTypes.SELECT,
                });

            allChats.map((user) => {
                return user.userId = user.fromUserId !== userId ? user.fromUserId : user.toUserId
            })

            usersList.map((user) => {
                const lastMessage = allChats.find((msg) => msg.userId === user.id);
                if (lastMessage != undefined) {
                    if (lastMessage.deleted_for_sender === 0 && lastMessage.deleted_for_recipient === 0)
                        return user.message = lastMessage
                }
            })

            usersList.sort((a, b) => {
                const createdAtA = a.message ? a.message.timestamps : 0; // If a message exists, get its createdAt timestamp; otherwise, use 0
                const createdAtB = b.message ? b.message.timestamps : 0; // If a message exists, get its createdAt timestamp; otherwise, use 0
                return createdAtB - createdAtA; // Sort in descending order
            });
            res.status(400).json([usersList]);
        }
        else {
            res.status(200).json([]);
        }

    } catch (error) {
        res.status(500).json('error' + error);
    }
}

const findUserChat = async (req, res) => {
    const userId = req.params.userId;

    try {
        const allChats = await sequelize.query('SELECT `id`, `members` FROM `chats`', {
            type: QueryTypes.SELECT,
        });

        const filteredChats = allChats.filter(chat =>
            JSON.stringify(chat.members).includes(userId)
        );
        res.status(200).json(filteredChats);

    } catch (error) {
        res.status(500).json('error' + error);
    }
}

const findChat = async (req, res) => {

    const { firstId, secondId } = req.params;
    try {
        const query = `
            SELECT id, members
            FROM chats
            WHERE JSON_CONTAINS(members, '[${firstId}]') AND JSON_CONTAINS(members, '[${secondId}]')
            LIMIT 1;
        `;

        const [chat, metadata] = await sequelize.query(query);
        res.status(200).json(chat);

    } catch (error) {
        res.status(500).json('error' + error);
    }
}

const deleteChat = async (req, res) => {
    try {
        const chatId = req.params.chatId;

        const deletedChat = await chatModel.destroy({ where: { id: chatId } });

        if (deletedChat === 0) {
            return res.status(404).json({ error: 'chat not found' });
        }

        return res.status(200).json({ message: 'Chat deleted successfully' });

    } catch (error) {
        return res.status(500).json({ error: 'Internal server error', error })
    }
}

const deleteChats = async (req, res) => {
    try {
        const { chatIds } = req.body;

        // Delete chats with the specified chatIds
        const deletedChatsCount = await chatModel.destroy({ where: { id: chatIds } });

        // Check if any chats were deleted
        if (deletedChatsCount === 0) {
            return res.status(404).json({ error: 'No chats found with the provided chatIds' });
        }

        return res.status(200).json({ message: 'Chats deleted successfully' });
    } catch (error) {
        console.error('Error deleting chats:', error);
        return res.status(500).json({ error: 'Internal server error', error });
    }
};

const createGroup = async (req, res) => {
    const { groupName, groupDescription } = req.body;
    try {
        if (req.files) {

            const file = req.files.file;
            const fileSize = req.files.file.size;
            const fileExtension = file.name.slice((file.name.lastIndexOf('.') - 1 >>> 0) + 2);

            const randomName = generateRandomString(20);
            if (fileSize < (5 * 1024 * 1024)) {
                let uploadPath = 'upload/images/' + randomName + '.' + fileExtension;
                file.mv(uploadPath, async function (err) {
                    if (err) {
                        return res.status(500).json({
                            success: 0,
                            message: 'Error uploading file',
                            error: err.message // Include error message in the response
                        });
                    }
                    const fileName = '/images/' + randomName + '.' + fileExtension;

                    const group = await Group.create({
                        user_id: req.userId,
                        name: groupName,
                        image: fileName,
                        description: groupDescription,
                    })
                    await GroupMember.create({
                        group_id: group.id,
                        user_id: group.user_id,
                        role:'owner'
                    });
                    req.io.join('group_' + group.id);

                    return res.status(200).json(group);
                });
            }
        } else {
            const group = await Group.create({
                name: groupName,
                user_id: req.userId,
                description: groupDescription,
            })

            await GroupMember.create({
                group_id: group.id,
                user_id: group.user_id,
                role:'owner'
            });
            req.io.join('group_' + group.id);

            return res.status(200).json(group);
        }
    } catch (error) {
        console.error('createGroup',error);
        return res.status(500).json(error);
    }
}
const updateGroup = async (req, res) => {
    const { groupName, groupDescription,id } = req.body;
    if (req.files) {

        const file = req.files.file;
        const fileSize = req.files.file.size;
        const fileExtension = file.name.slice((file.name.lastIndexOf('.') - 1 >>> 0) + 2);

        const randomName = generateRandomString(20);
        if (fileSize < (5 * 1024 * 1024)) {
            let uploadPath = 'upload/images/' + randomName + '.' + fileExtension;
            file.mv(uploadPath, async function (err) {
                if (err) {
                    return res.status(500).json({
                        success: 0,
                        message: 'Error uploading file',
                        error: err.message // Include error message in the response
                    });
                }
                const fileName = '/images/' + randomName + '.' + fileExtension;
                const group =  await Group.update({
                    name: groupName,
                    image: fileName,
                    description: groupDescription,
                },{where:{id}})

                return res.status(200).json(group);
            });
        }
    } else {
        const group = await Group.update({
            name:groupName,
            description: groupDescription,
        },{where:{id}})

        return res.status(200).json(group);
    }
}

const getChatIdsByMemberId = async (req, res) => {
    try {
        const { memberId } = req.params;

        // Use Sequelize query instead of findAll to execute custom SQL query
        const existingChat = await sequelize.query(
            'SELECT `id`, `members` FROM `chats` WHERE JSON_CONTAINS(`members`, :memberId)',
            {
                replacements: { memberId: memberId },
                type: Sequelize.QueryTypes.SELECT,
                model: chatModel,
            }
        );

        // Extract chat IDs from the retrieved chats
        const chatIds = existingChat.map(chat => chat.id);

        res.json({ success: true, chatIds });
    } catch (error) {
        console.error('Error fetching chat IDs:', error);
        res.status(500).json({ success: false, error: 'Internal server error', error });
    }
};

const generateRandomString = (length) => {
    const characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    let result = '';
    for (let i = 0; i < length; i++) {
        result += characters.charAt(Math.floor(Math.random() * characters.length));
    }
    return result;
};



module.exports = {
    createChat,
    findUserChat,
    findChat,
    getAllChats,
    deleteChat,
    deleteChats,
    getChatIdsByMemberId,
    getChatUserLists,
    getAllData,
    updateGroup,
    createGroup
}
