const { Op } = require("sequelize");
const messageModel = require("../models/Message");
const CryptoJS = require("crypto-js");
const moment = require("moment");


// Generate a random key for AES-256 encryption
const key = "mySecretKey123"; // 32 bytes = 256 bits

// Encrypt function
const encryptText = (text, key) => {
    const encrypted = CryptoJS.AES.encrypt(text, key).toString();
    return encrypted;
}

const createMessage = async (req, res) => {

    const { toUserId, text, status } = req.body
    try {
        const date = new Date()
            date.setHours(date.getHours() + 5); // Add 5 hours for IST
            date.setMinutes(date.getMinutes() + 30);
        
        if (req.files) {
            
            const file = req.files.file;
            const fileSize = req.files.file.size;
            const fileExtension = file.name.slice((file.name.lastIndexOf('.') - 1 >>> 0) + 2);
                        
            const randomName = generateRandomString(20);
            if (fileSize < (5 * 1024 * 1024)) {
                let uploadPath = 'upload/images/' + randomName+'.'+fileExtension;
                file.mv(uploadPath, async function (err) {
                    if (err) {
                        return res.status(500).json({
                            success: 0,
                            message: 'Error uploading file',
                            error: err.message // Include error message in the response
                        });
                    }
                    const fileName = '/images/' + randomName+'.'+fileExtension;
                    const encryptedText = encryptText(text, key);
                    const message = await messageModel.create({
                        fromUserId: req.userId,
                        toUserId: toUserId,
                        text: encryptedText,
                        file_url: fileName,
                        file_name:file.name,
                        file_size:file.size,
                        file_type:file.mimetype,
                        createAt:date.toISOString().split('T')[0]
                    });

                    const messageData = {
                        id: message.id,
                        toUserId: message.toUserId,
                        fromUserId: message.fromUserId,
                        text: encryptedText,
                        deleted_for_sender:0,
                        deleted_for_recipient:0,
                        file_url: message.file_url,
                        file_name: message.file_name,
                        file_size: message.file_size,
                        file_type: message.file_type,
                        status: message.status,
                        createdAt: message.createdAt,
                        updatedAt: message.updatedAt,
                    }
                    return res.status(200).json(messageData);
                });
            }
        } else {
            const encryptedText = encryptText(text, key);
            const message = await messageModel.create({
                fromUserId: req.userId,
                toUserId: toUserId,
                text: encryptedText,
                createAt:date.toISOString().split('T')[0]
            });

            const messageData = {
                id: message.id,
                toUserId: parseInt(message.toUserId),
                fromUserId: message.fromUserId,
                text: encryptedText,
                file_url: null,
                deleted_for_sender:0,
                deleted_for_recipient:0,
                status: message.status,
                createdAt: message.createdAt,
                updatedAt: message.updatedAt,
            }
            res.status(200).json(messageData);
        }
    } catch (error) {
        res.status(500).json('error' + error)
    }
}

const getMessages = async (req, res) => {

    const { chatId } = req.params
    try {

        const messages = await messageModel.findAll({
            where: {
                [Op.or]: [{ fromUserId: req.userId, toUserId: chatId }, { toUserId: req.userId, fromUserId: chatId }],
                isDeletedForEveryone:0
            },
            // where: {isDeletedForEveryone:0},
        });

        // Decrypt each message before sending it in the response
        // const decryptedMessages = messages.map(message => {
        //     return {
        //         id: message.id,
        //         chatId: message.chatId,
        //         senderId: message.senderId,
        //         text: message.text, // Decrypt the message before sending it
        //         url: message.url,
        //         createdAt: message.createdAt,
        //         updatedAt: message.updatedAt,
        //         status: message.status,
        //     };
        // });

        res.status(200).json(messages);

    } catch (error) {
        res.status(500).json(error.message);
    }
}

const getMediaMessages = async (req, res) => {

    const { chatId,type } = req.params
    try {
        if (type=='group') {
            const messages = await messageModel.findAll({
                where: {
                    group_id:chatId,
                    file_url: {
                        [Op.ne]: null
                    },
                    isDeletedForEveryone:0
                },
            });
    
            res.status(200).json(messages);
        } else {
            const messages = await messageModel.findAll({
                where: {
                    [Op.or]: [{ fromUserId: req.userId, toUserId: chatId }, { toUserId: req.userId, fromUserId: chatId }],
                    file_url: {
                        [Op.ne]: null
                    },
                    isDeletedForEveryone:0
                },
            });
            res.status(200).json(messages); 
        }

    } catch (error) {
        res.status(500).json(error.message);
    }
}

const updateMessagesStatus = async (req, res) => {
    try {
        // Parse array of message IDs from request body
        const { messageIds } = req.body;
        const { status } = req.body;

        // Update status of messages with IDs in the array
        const updatedRows = await messageModel.update(
            { status },
            { where: { id: messageIds } }
        );

        // Check if any rows were affected by the update operation
        if (updatedRows[0] === 0) {
            // If no rows were affected, none of the message IDs were found
            return res.status(404).json({ error: "No messages found with the provided IDs" });
        }

        // Return success response
        res.status(200).json({ success: true });
    } catch (error) {
        res.status(500).json({ error: error.message });
    }
};


const updateMessageStatusByChatId = async (req, res) => {
    const { chatId } = req.params;
    const { status } = req.body;

    try {

        const updatedRows = await messageModel.update(
            { status },
            { where: { chatId: chatId } }
        );


        // Check if any rows were affected by the update operation
        if (updatedRows[0] === 0) {
            // If no rows were affected, the message with the given ID was not found
            return res.status(404).json({ error: "Message not found" });
        }

        // Return success response
        res.status(200).json({ success: true });
    } catch (error) {
        res.status(500).json({ error: error.message });
    }
}

const updateStatusAllMessages = async (req, res) => {
    const { chatId, senderId } = req.params;

    try {
        // Update status of all messages where chatId and senderId match
        const updatedMessages = await messageModel.update(
            { status: "read" }, // Update status to "read"
            { where: { chatId, senderId } } // Filter by chatId and senderId
        );

        res.status(200).json({ message: "Status updated for all matching messages" });
    } catch (error) {
        res.status(500).json({ error: error.message });
    }
}

const updateStatusMessages = async (req, res) => {
    const { toUserId } = req.params;
    const { status } = req.body;

    try {
        // Update status of all messages where chatId and senderId match
        const updatedMessages = await messageModel.update(
            { status: status }, // Update status to "read"
            { where: { fromUserId:toUserId,toUserId:req.userId} } // Filter by chatId and senderId
        );

        res.status(200).json({ message: "Status updated for all matching messages" });
    } catch (error) {
        res.status(500).json({ error: error.message });
    }
}
const deleteMessage = async (req, res) => {
    console.log('messageid', req.params)
    try {
        const messageId = req.params.messageId;
        const isFor = req.body.isFor;
        const userId = req.userId;
        console.log('userser id',userId);
        if (!messageId || !isFor) {
            return res.status(400).json({ error: 'Missing required parameters' });
        }

        let updateMessageStatus;
        if (isFor === "everyone") {
            const isDeletedForEveryOne = req.body.isDeletedForEveryOne;
            updateMessageStatus = await messageModel.update(
                { isDeletedForEveryOne },
                { where: { id: messageId } }
            );
        } else if (isFor === "me") {
            try {
                const message = await messageModel.findByPk(messageId);
            
                if (!message) {
                  return res.status(404).send('Message not found');
                }
            
                if (message.fromUserId === parseInt(userId)) {
                  message.deleted_for_sender = 1;
                }
                if (message.toUserId === parseInt(userId)) {
                  message.deleted_for_recipient = 1;
                }
            
                await message.save();
                return res.status(200).json({ message: 'Message deleted successfully' });
            } catch (err) {
                res.status(500).send(err);
            }
        } else {
            return res.status(400).json({ error: 'Invalid value for isFor parameter' });
        }
        console.log('updateMessageStatus --------', updateMessageStatus)
        if (updateMessageStatus[0] === 0) { // Checking the number of affected rows
            return res.status(404).json({ error: 'Message not found' });
        }

        return res.status(200).json({ message: 'Message deleted successfully' });
    } catch (error) {
        console.error('Error deleting message:', error);
        return res.status(500).json({ error: 'Internal server error' });
    }
}

// const deleteMessage = async (req, res) => {

//     try {

//         const messageId = req.params.messageId;

//         const deletedMessage = await messageModel.destroy({ where: { id: messageId } });

//         if (deletedMessage === 0) {
//             return res.status(400).json({ error: 'message not found' });
//         }

//         return res.status(200).json({ message: 'Message deleted successfully...' });

//     } catch (error) {
//         return res.status(500).json({ error: 'Internal server error', error })
//     }
// }

const deleteMessageByChatId = async (req, res) => {

    try {

        const chatId = req.params.chatId;

        const deletedMessageByChatId = await messageModel.destroy({ where: { chatId: chatId } });

        if (deletedMessageByChatId === 0) {
            return res.status(400).json({ error: 'message not found' });
        }

        return res.status(200).json({ message: 'Message deleted successfully...' })

    } catch (error) {
        return res.status(500).json({ error: 'Internal server error', error })
    }
}

const deleteMessagesByChatIds = async (req, res) => {
    try {
        const { chatIds } = req.body;

        // Delete chats with the specified chatIds
        const deletedMessagesCount = await messageModel.destroy({ where: { chatId: chatIds } });

        // Check if any chats were deleted
        if (deletedMessagesCount === 0) {
            return res.status(404).json({ error: 'No chats found with the provided chatIds' });
        }

        return res.status(200).json({ message: 'messages deleted successfully' });
    } catch (error) {
        console.error('Error deleting messages:', error);
        return res.status(500).json({ error: 'Internal server error', error });
    }
};
const generateRandomString = (length) => {
    const characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    let result = '';
    for (let i = 0; i < length; i++) {
      result += characters.charAt(Math.floor(Math.random() * characters.length));
    }
    return result;
};
module.exports = {
    createMessage,
    getMessages,
    updateStatusAllMessages,
    deleteMessage,
    deleteMessageByChatId,
    deleteMessagesByChatIds,
    updateMessageStatusByChatId,
    updateMessagesStatus,
    updateStatusMessages,
    getMediaMessages
}